/***************************************************************************
                          connector.c  -  description
                             -------------------
    begin                : Mon Mar 12 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


 #include "connector.h"
 #include "handlers.h"
 #include "contacts.h"

 /*
 connector_mod_gen_arg

 Generates argument for the function activated by the thread
 create call.
 */
 void *connector_mod_gen_arg(int accepted_sock, struct sockaddr_in *remote_addr, int sin_size)
 {
 		connector_arg *arg = (connector_arg *) malloc(sizeof(connector_arg));
 		
 		if (!arg)
 		{
 			close(accepted_sock);
 			return NULL;
 		}
 		
 		arg->sock = accepted_sock;
 		arg->remote_addr = *remote_addr;
 		arg->sin_size = sin_size;
 		
 		return ((void *) arg);
 }

 /*
 connector_mod

 Calls the connector function with the correct parameters.
 */
 void *connector_mod(void *inp)
 {
 		connector_arg *arg = (connector_arg *) inp;
 		int s;
 		struct sockaddr_in remote_addr;
 		int sin_size;
 		
 		if (!arg) return NULL;
 		
 		s = arg->sock;
 		remote_addr = arg->remote_addr;
 		sin_size = arg->sin_size;
 		
 		free(inp);
 		
 		return ((void *) connector(s, &remote_addr, sin_size));
 }


 /*
 connector

 Main function for the connector application.
 */
 int connector(int s, struct sockaddr_in *remote_addr, int sin_size)
 {
 		link_env env; // The link's environment
 		fd_set readfds, exceptfds;
 		struct timeval tv;
 		time_t last_keepalive_time; // Time last keepalive sent
 		
 		announce_connection(remote_addr);
  		
 		env.s_client = s;
 		env.b_active = 0; // We are not yet connected to the network
 		//env.b_writing = 0;
 		env.b_halt = 0;
 		env.data = NULL;
 		init_contacts(&env);
 		
 		last_keepalive_time = time(NULL);
	 		
 		while (!env.b_halt)
 		{
 			
 			tv.tv_sec = SELECT_WAIT_SEC;
 			tv.tv_usec = SELECT_WAIT_USEC;
			
 			if (env.b_active)
 			{
 				if ((time(NULL)-last_keepalive_time) >= TIME_BETWEEN_KEEPALIVES_SEC)
 				{
 					icq_KeepAlive(&env.link);
 					last_keepalive_time=time(NULL);
 				}
 				icq_Main(&env.link);
 				/*icq_SendContactList(&env.link);*/
 			}
 			
#ifdef DEBUG2
			fprintf(stderr, "Before selecting\n");
#endif

 			
  		FD_ZERO(&readfds);
		  FD_SET(env.s_client, &readfds);
  		FD_ZERO(&exceptfds);
		  FD_SET(env.s_client, &exceptfds);
		  select(env.s_client+1, &readfds, NULL, &exceptfds, &tv);

#ifdef DEBUG2
			fprintf(stderr, "Before isset\n");
#endif

  		if(FD_ISSET(env.s_client, &exceptfds))
  				env.b_halt = 1;
  		else		
  			if(FD_ISSET(env.s_client, &readfds))
 		 		{
  				if ((env.data = receive_message(env.s_client)))
  					handle_message(&env);
  				else
  					env.b_halt = 1;
	  		}
 		}
 		
 		clean_env(&env);
 		
 		return 0;
 }
