/***************************************************************************
                          contacts.c  -  description
                             -------------------
    begin                : Thu Apr 12 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "contacts.h"

 /*
 This file contains functions that handle the contacts section of the
 environment.
 */

 /*
 push_contact_list

 Pushes contact list to client.
 */

 void push_contact_list(link_env *penv)
 {
 		char message[4+4+4+4+MAX_CNAME_SIZE+1];
 		centry *current;

#ifdef DEBUG
		fprintf(stderr, "%lu: Pushing contacts.\n", penv->link.icq_Uin);
#endif
 		
#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tPushing contacts:\n", penv->link.icq_Uin);
#endif
		
 		
 		for (current=penv->contacts.first; current; current = current->next)
 		{

#ifdef DEBUG
		fprintf(stderr, "%lu: Pushing contact (%lu, %s) to user\n", penv->link.icq_Uin, current->uin, current->name);
#endif
 		
#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tPushing contact (%lu, %s)\n", penv->link.icq_Uin, current->uin, current->name);
#endif
 				(*(unsigned long *) message) = htonl(4+4+4+4+strlen(current->name)+1);
 				(*(unsigned long *) (message+4)) = htonl(STOC(PSH_CNTCT));
 				(*(unsigned long *) (message+8)) = htonl(current->uin);
 				(*(unsigned long *) (message+12)) = htonl(strlen(current->name)+1);
 				strcpy(message+16, current->name);
 			
 				if (send_message(penv->s_client, message, 4+4+4+4+strlen(current->name)+1) == -1)
 				{
 					penv->b_halt = 1;
 					return;
 				}
 				
 				icq_ContactAdd(&penv->link, current->uin);
 		}
 		
 		return;
 }



 /*
 init_contacts

 Initializes contacts
 */

 void init_contacts(link_env *penv)
 {
 		cl_init(&penv->contacts);
 		penv->v_contacts = 0;
 		return;
 }

 /*
 start_contact_list

 Brings contact list from database
 */

 void start_contact_list(link_env *penv)
 {
 		 if (penv->v_contacts)
 		 		return;
 		 		
#ifdef DEBUG
		fprintf(stderr, "Going to get contact list.\n");
#endif 		
 		
 		 get_contact_list(penv->link.icq_Uin, &penv->contacts);

#ifdef DEBUG
		fprintf(stderr, "Finished getting contact list.\n");
#endif 		
 		
 		  		
#ifdef DEBUG
		fprintf(stderr, "%lu: Got contact list from file.\n", penv->link.icq_Uin);
#endif 		
 		 penv->v_contacts = 1;
 		 push_contact_list(penv);
 		 return;
 }

 /*
 term_contact_list

 Writes contact list to database and clears it
 */

 void term_contact_list(link_env *penv)
 {
 		 if (!(penv->v_contacts))
 		 		return;
 		 		
 		 put_contact_list(penv->link.icq_Uin, &penv->contacts);
 		 cl_clean(&penv->contacts);
 		 cl_init(&penv->contacts);
 		 penv->v_contacts = 0;
 		 return;
 }

 /*
 another_contact

 Adds a contact to contact list.
 */

 void another_contact(link_env *penv, char *cname, unsigned long cuin)
 {
 	centry *item;
 	
 	if (!(penv->v_contacts))
 		return;
 		
 	item = cl_find(&penv->contacts, NULL, cuin);
 	
 	if (item) // If the uin is already there, we just change the name
 	{
 		char *temp = (char *) malloc(strlen(cname)+1);
 		
 		if (!temp)
 			return;
 			
 		if (item->name)
 			free(item->name);
 		
 		item->name = temp;
 		strcpy(item->name, cname);
 	}
 	else
 		cl_add(&penv->contacts, cname, cuin);
 		
 	return;
 }

 /*
 remove_contact

 Removes a contact from the contact list.
 */

 void remove_contact(link_env *penv, unsigned long cuin)
 {
 	if (!(penv->v_contacts))
 		return;
 		
 	cl_delete(&penv->contacts, NULL, cuin);
 	
 	return;
 }


