/***************************************************************************
                          db.c  -  description
                             -------------------
    begin                : Wed Apr 11 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "db.h"

 /*
 This file implements all database features.
 */

 /*
 compose_db_path

 Assigns the path of the database into the given string.
 */

 void compose_db_path(char *result, const char *file)
 {
  strcpy(result, HOME_DIR);
 	strcat(result, DB_PATH);
 	
 	if (file)
 		strcat(result, file);
 	
 	return;
 }


 /*
 initialize_db

 Makes sure database subdirectory exists and creates it otherwise.
 Returns zero if successful, otherwise returns the value of errno.
 */

 int initialize_db()
 {
 	int e_val=0; // error value
 	char db_name[DB_PATH_LEN+1]; // The name of the directory
 	 	
 	compose_db_path(db_name, NULL);

 	if (mkdir(db_name, DB_PERMISSIONS) == -1)
 	{
 		e_val = errno;
 		 		 		
 		if (e_val == EEXIST)
 			e_val = 0;
 	}
 	
 	return e_val;
 	
 }


 /*
 get_contact_list

 Takes the contact list from the database and puts it into
 the given contact list.
 */

 void get_contact_list(unsigned long uin, clist *list)
 {
 		char file[UIN_LEN+DB_EXT_LEN+1]; // The name of the file
 		char db_entry_path[DB_PATH_LEN+UIN_LEN+DB_EXT_LEN+1]; // The string that contains the path of the database entry
 		FILE *db_entry; // The database entry of the given uin
 		
 		cl_init(list);
	
 		sprintf(file, "%lu%s", uin, DB_EXT);
 		
#ifdef DEBUG
		fprintf(stderr, "file: %s\n", file);
#endif
 		
 		compose_db_path(db_entry_path, file);
 		
#ifdef DEBUG
		fprintf(stderr, "db_entry_path: %s\n", db_entry_path);
#endif
 		
 		db_entry = fopen(db_entry_path, "r");
 		 		
#ifdef DEBUG
		fprintf(stderr, "Opened database entry. Null pointer? %c\n", (db_entry)?'n':'y');
#endif
 		
 		if (!db_entry)
 		{
#ifdef DEBUG
			fprintf(stderr, "Database entry is null.\n");
#endif
 			return;
 		}

#ifdef DEBUG
			fprintf(stderr, "Database entry is not null.\n");
#endif
 		
 		cl_read(list, db_entry);
 		 		
 		fclose(db_entry);
 		
		return;
 }


 /*
 put_contact_list

 Writes contact list to database.
 */

 void put_contact_list(unsigned long uin, clist *list)
 {
 		char file[UIN_LEN+DB_EXT_LEN+1]; // The name of the file
 		char db_entry_path[DB_PATH_LEN+UIN_LEN+DB_EXT_LEN+1]; // The string that contains the path of the database entry
 		FILE *db_entry; // The database entry of the given uin
 		
 		sprintf(file, "%lu%s", uin, DB_EXT);
 		compose_db_path(db_entry_path, file);
 		
 		db_entry = fopen(db_entry_path, "w");
 		if (!db_entry)
 			return;
 		
 		cl_write(list, db_entry);
 		
 		fclose(db_entry);
 		
		return;
 }



