/***************************************************************************
                          handlers.c  -  description
                             -------------------
    begin                : Tue Mar 13 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 #include "handlers.h"

 /*
 Implementations of handler functions
 */


 /*
 Following are handlers for events on the icq socket
 */

 /*
 icq_handle_logged_in

 ICQ informed us we're logged in
 */

 void icq_handle_logged_in(struct icq_link *link)
 {
 		link_env *penv = (link_env *) ENV(link); // Link's environment
 		char buf[8];
 		unsigned long *pL;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Connection established!\n", link->icq_Uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tConnected to icq network.\n", link->icq_Uin);
#endif

 		 		
 		pL = (unsigned long *) buf;
 		
 		*pL = htonl(8);
 		pL++;
 		*pL = htonl(STOC(SV(LOGIN_REQ)));
 		
 		if (send_message(penv->s_client, buf, 8) == -1)
 			penv->b_halt = 1;

#ifdef DEBUG
		fprintf(stderr, "%lu: Starting contact list...\n", link->icq_Uin);
#endif
 			
 		start_contact_list(penv);
 		
 		// Sending contact list to server
		icq_SendContactList(&penv->link);

#ifdef DEBUG
		fprintf(stderr, "%lu: Contact list started.\n", link->icq_Uin);
#endif
 		
 		return;		
 		
 }

  /*
 icq_handle_not_logged_in

 ICQ informed us we can't login
 */

 void icq_handle_not_logged_in(struct icq_link *link)
 {
 		link_env *penv = (link_env *) ENV(link); // Link's environment
 		char buf[8];
 		unsigned long *pL;
 		 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Connection refused!\n", link->icq_Uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tConnection to the ICQ server refused.\n", link->icq_Uin);
#endif

 		
 		pL = (unsigned long *) buf;
 		
 		*pL = htonl(8);
 		pL++;
 		*pL = htonl(STOC(SX(LOGIN_REQ)));
 		
 		if (send_message(penv->s_client, buf, 8) == -1)
 			penv->b_halt = 1;
 		
 		return;		
 		
 }

  /*
 icq_handle_disconnect

 ICQ informed us we're disconnected
 */

 void icq_handle_disconnect(struct icq_link *link)
 {
 		link_env *penv = (link_env *) ENV(link); // Link's environment
 		char buf[8];
 		unsigned long *pL;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Disconnected from server.\n", link->icq_Uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tDisconnected from ICQ server.\n", link->icq_Uin);
#endif
 		 		
 		pL = (unsigned long *) buf;
 		
 		*pL = htonl(8);
 		pL++;
 		*pL = htonl(STOC(INF_DISCNCT));
 		
 		if (send_message(penv->s_client, buf, 8) == -1)
 			penv->b_halt = 1;
 		
 		//penv->b_active = 0;
		
 		// Attempting to login again
		icq_Connect(&penv->link, ICQ_HOST, ICQ_PORT);
		icq_Login(&penv->link, STATUS_ONLINE);
 		
 		// The following two lines have been disabled in order to
 		// attempt to try and reconnect in case of disconnection.
 		//icq_Done(link);
 		
 		return;		
 }

  /*
 icq_handle_recv_message

 We got a message on ICQ
 */

 void icq_handle_recv_message(struct icq_link *link, unsigned long uin, unsigned char hour,
 															unsigned char minute, unsigned char day, unsigned char month,
 															unsigned short year, const char *msg)
 {
 		link_env *penv = (link_env *) ENV(link);
 		unsigned long length;
 		char *buf, *pC;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Received message. [%s]\n", link->icq_Uin, msg);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tReceived message from %lu:\n{begin}\n%s\n{end}\n", link->icq_Uin, uin, msg);
#endif

 		
 		length = 4+4+4+4+strlen(msg)+1;
 		while (!(pC = malloc(length)))
 			sleep(1);
 			
 		buf = pC;
 			
 		*((unsigned long *) pC) = htonl(length);
 		pC+=4;
 		
 		*((unsigned long *) pC) = htonl(STOC(RECV_MSG));
 		pC+=4;

 		*((unsigned long *) pC) = htonl(uin);
 		pC+=4;
 		
 		*((unsigned long *) pC) = htonl(strlen(msg)+1);
 		pC+=4;
 		
 		strcpy(pC, msg);
 		
 		
 		if (send_message(penv->s_client, buf, length) == -1)
 			penv->b_halt = 1;
 		
 		free(buf);
 		
 		return;
 }


 /*
 icq_handle_update_status

 Handles an update and the status of a user in the contact list
 */
 void icq_handle_update_status(struct icq_link *link, unsigned long uin, unsigned long status)
 {
 		link_env *penv = (link_env *) ENV(link);
 		char buf[16], *pC=buf;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: User %lu changed status to %lu.\n", link->icq_Uin, uin, status);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tUser %lu changed status to %lu.\n", link->icq_Uin, uin, status);
#endif
 		
 		*((unsigned long *) pC) = htonl(16);
 		pC+=4;
 		
 		*((unsigned long *) pC) = htonl(STOC(UPDATE_USR));
 		pC+=4;

 		*((unsigned long *) pC) = htonl(uin);
 		pC+=4;
 		
 		*((unsigned long *) pC) = htonl(status);
 		
 		if (send_message(penv->s_client, buf, 16) == -1)
 			penv->b_halt = 1;
 		
 		return;
 }

 /*
 icq_handle_user_online

 Handles an event of user coming online
 */
 void icq_handle_user_online(struct icq_link *link, unsigned long uin, unsigned long status,
 															unsigned long ip, unsigned short port, unsigned long real_ip,
 															unsigned char tcp_flag)
 {
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: User %lu came online.\n", link->icq_Uin, uin);
#endif
		
#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tUser %lu came online.\n", link->icq_Uin, uin);
#endif
 		
 		icq_handle_update_status(link, uin, STATUS_ONLINE);
 		return;
 }


 /*
 icq_handle_user_offline

 Handles an event of user going offline
 */
 void icq_handle_user_offline(struct icq_link *link, unsigned long uin)
 {
#ifdef DEBUG
		fprintf(stderr, "%lu: User %lu went offline.\n", link->icq_Uin, uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tUser %lu went offline.\n", link->icq_Uin, uin);
#endif
 		
 		icq_handle_update_status(link, uin, STATUS_OFFLINE);
 		return;
 }


 /*
 icq_handle_srv_ack

 For debug reasons
 */
 void icq_handle_srv_ack(struct icq_link *link, unsigned short seq)
 {
#ifdef DEBUG
		fprintf(stderr, "%lu: Got SrvAck %d.\n", link->icq_Uin, seq);
#endif
 		
 		return;
 }


 /* ============================================= */



/*
Following are handlers for messages received from the client
*/




 /*
 handle_login_req

 Client asks to login
 */
 void handle_login_req(link_env *penv)
 {
 		login_req req;
 		char *pC = penv->data;
 		
 		if (penv->b_active)
 			return; // you need to disconnect before you try to login again
 		
 		pC += 4; // The first field is message length which isn't interesting
 		
 		req.type = ntohl(* (unsigned long *) pC);
 		pC += 4;
 		
 		req.uin = ntohl(* (unsigned long *) pC);
 		pC += 4;
 		
 		req.password.length = ntohl(* (unsigned long *) pC);
 		pC += 4;
 		
 		req.password.text = (char *) pC;
 		pC += req.password.length;
 		
 		req.nick.length = ntohl(* (unsigned long *) pC);
 		pC += 4;
 		
 		req.nick.text = (char *) pC;
 		pC += req.nick.length;
 		
 		
#ifdef DEBUG
		fprintf(stderr, "Initializing connection. UIN: %lu, Password: ***, Nick: %s\n", req.uin, req.nick.text);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, ">>Initializing connection: UIN=%lu\tPassword=********\tNick=%s\n", req.uin, req.nick.text);
#endif

 		
  	icq_Init(&penv->link, req.uin, req.password.text, req.nick.text);
  	
  	ENV((&(penv->link))) = (char *) penv; // Putting a link to the environment
  	penv->b_active = 1;

	  penv->link.icq_Logged = &icq_handle_logged_in;
	  penv->link.icq_Disconnected = &icq_handle_disconnect;
	  penv->link.icq_RecvMessage = &icq_handle_recv_message;
	  penv->link.icq_WrongPassword = &icq_handle_not_logged_in;
	  penv->link.icq_InvalidUIN = &icq_handle_not_logged_in;
	  penv->link.icq_UserStatusUpdate = &icq_handle_update_status;
	  penv->link.icq_UserOnline = &icq_handle_user_online;
	  penv->link.icq_UserOffline = &icq_handle_user_offline;
	  penv->link.icq_SrvAck = &icq_handle_srv_ack;
		

		icq_Connect(&penv->link, ICQ_HOST, ICQ_PORT);
		icq_Login(&penv->link, STATUS_ONLINE);
		
		return;
 }


 /*
 handle_disconnect

 Client wants to disconnect
 */

 void handle_disconnect(link_env *penv)
 {
 		if (!(penv->b_active))
 			return; // You are already disconnected!
 			
#ifdef DEBUG
		fprintf(stderr, "%lu: Terminating connection.\n", penv->link.icq_Uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tConnection terminated.\n", penv->link.icq_Uin);
#endif
 		
 		
 		penv->b_active = 0;
 		icq_Logout(&penv->link);
 		icq_Disconnect(&penv->link);
 		icq_Done(&penv->link);
 		
 		close(penv->s_client); // Closing socket
 		penv->b_halt = 1;
 		
 		return;
 }

 /*
 handle_send_msg

 The client wants to send a message
 */
 void handle_send_msg(link_env *penv)
 {
 		char *pC = penv->data;
 		unsigned long uin;
 		
 		pC += 8; // We don't care about the length and type fields
 		uin = ntohl(* (unsigned long *) pC);
 		pC += 8;

#ifdef DEBUG
		fprintf(stderr, "%lu: Sending message to %lu [%s]\n", penv->link.icq_Uin, uin, pC);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tSending message to %lu:\n{begin}\n%s\n{end}\n", penv->link.icq_Uin, uin, pC);
#endif
 		
 		icq_SendMessage(&penv->link, uin, pC, ICQ_SEND_BESTWAY);
 		
 		return;
 }

 /*
 handle_add_usr

 The client wants a uin to be added into the contact list
 */
 void handle_add_usr(link_env *penv)
 {
 		char *pC = penv->data;
 		unsigned long uin;
 		
		
 		uin = ntohl(* (unsigned long *) (pC + 8));
 		pC += 16;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Adding user %lu to contact list.\n", penv->link.icq_Uin, uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tAdding user (%lu, %s) to contact list.\n", penv->link.icq_Uin, uin, pC);
#endif

 		
 		icq_ContactAdd(&penv->link, uin);
 		icq_SendNewUser(&penv->link, uin);
 		
 		another_contact(penv, pC, uin);
 		
 		return;
 }


 /*
 handle_del_usr

 The client wants a uin to be added into the contact list
 */
 void handle_del_usr(link_env *penv)
 {
 		char *pC = penv->data;
 		unsigned long uin;
 		
 		uin = ntohl(* (unsigned long *) (pC + 8));
 		pC += 16;
 		
#ifdef DEBUG
		fprintf(stderr, "%lu: Removing user %lu from contact list.\n", penv->link.icq_Uin, uin);
#endif

#ifdef DO_LOG
		fprintf(LOG_FILE, "%lu\tRemoving user %lu from contact list.\n", penv->link.icq_Uin, uin);
#endif
 		
 		icq_ContactRemove(&penv->link, uin);
 		
 		remove_contact(penv, uin);
 		
 		return;
 }



 /* ============================================= */

 /*
 handle_message

 Handles a message received from the client
 */
 int handle_message(link_env *penv)
 {
 		unsigned long type;
 		
 		if (!(penv->data))
 			return 1;
 		
 		type = message_type(penv->data);
 		
#ifdef DEBUG
		fprintf(stderr, "Received message of type %lu from client.\n", type);
#endif
	
 		switch (type)
 		{
 			case (CTOS(LOGIN_REQ)):
 				handle_login_req(penv);
 				break;
 				
 			case (CTOS(SEND_MSG)):
 				handle_send_msg(penv);
 				break;
 				
 			case (CTOS(ADD_USR)):
 				handle_add_usr(penv);
 				break;

 			case (CTOS(DEL_USR)):
 				handle_del_usr(penv);
 				break;

 			case (CTOS(DISCONNECT)):
 				handle_disconnect(penv);
 				break;
 			}
 			
 			if (penv->data)
 			{
 				clean_buf(penv->data);
 				penv->data = NULL;
 			}
 			
 			return 0;
 }





 /* ============================================= */

 /*
 clean_env

 Cleans the environment when we are finish
 */

 int clean_env(link_env *penv)
 {
 		if (penv->data)
 		{
 			clean_buf(penv->data);
 			penv->data = NULL;
 		}
 			
 		close(penv->s_client);
 		
 		if (penv->b_active)
 		{
 			icq_Logout(&penv->link);
 			icq_Disconnect(&penv->link);
 			icq_Done(&penv->link);
 			penv->b_active = 0;
 		}

 		term_contact_list(penv); 		
 		
 		return 0;
 		
 }

