#ifndef __PROTOCOL_H__
#define __PROTOCOL_H__

/***************************************************************************
                          protocol.h  -  description
                             -------------------
    begin                : Sat Mar 10 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 /*
 This file specifies the protocol for sent and received messages in the csicq
 project.
 */

// Messages from client to server
 #define CTOS_MSG 100
 #define CTOS(x) (CTOS_MSG+(x))
// Messages from server to client
 #define STOC_MSG 200
 #define STOC(x) (STOC_MSG+(x))

// C->S Messages
#define LOGIN_REQ 	1
#define SEND_MSG 		2
#define ADD_USR			3
#define DEL_USR			4
#define DISCONNECT	5


// S->C Messages
#define LOGIN_AUTH	1
#define RECV_MSG		2
#define UPDATE_USR	3
#define INF_DISCNCT	4
#define PSH_CNTCT		7

// Server authorizes request
#define S_AUTH 			50
#define SV(x)				(S_AUTH+(x))

// Server refuses request
#define S_REFUSE		60
#define SX(x)				(S_REFUSE+(x))


typedef struct { // A struct to hold text fields of messages
	unsigned long length;
	char *text;
} text_field;


/*
Client -> Server requests

Following are definitions of structs that are used to hold requests from client
to server.
*/

typedef struct { // A struct to hold login requests
	unsigned long type;
	unsigned long uin;
	text_field password;
	text_field nick;
} login_req;

typedef struct { // A struct to hold send requests
	unsigned long type;
	unsigned long uin;
	text_field message;
} send_req;

typedef struct { // A struct to hold add user requests
	unsigned long type;
	unsigned long uin;
	text_field name;
} add_usr_req;

typedef struct { // A struct to hold remove user requests
	unsigned long type;
	unsigned long uin;
} rem_usr_req;

typedef struct { // A struct to hold disconnection requests
	unsigned long type;
} disconnect_req;


/*
Server -> Client messages

Following are definitions of structs that are used to hold messages from the server
to the client.
*/

typedef struct { // A struct to hold authorize login message
	unsigned long type;
} auth_login;

typedef struct { // A struct to hold message received message
	unsigned long type;
	unsigned long from_uin;
	text_field message;
} recv_message;

typedef struct { // A struct to hold update status message
	unsigned long type;
	unsigned long uin;
	unsigned long status;
} update_usr_status;

typedef struct { // A struct to hold inform of disconnection message
	unsigned long type;
} inform_disconnect;



/*
Following is a general union of all message types.
*/
typedef union { // a union to hold all possible messages
	/* C->S */
	login_req 					l_req;
	send_req						s_req;
	add_usr_req					au_req;
	rem_usr_req					ru_req;
	disconnect_req			d_req;
	
	/* S->C */
	auth_login					auth_msg;
	recv_message				recv_msg;
	update_usr_status		update_msg;
	inform_disconnect		inform_msg;
	
} generic_msg;


/*
Following are definitions concerning the icq network
*/

#define ICQ_HOST "icq.mirabilis.com"
#define ICQ_PORT 4000



#endif	//#ifndef __PROTOCOL_H__
