/***************************************************************************
                          services.c  -  description
                             -------------------
    begin                : Mon Mar 12 2001
    copyright            : (C) 2001 by Zvika Brakerski, Asaf Koren
    email                : zvika@eng.tau.ac.il
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


 #include "services.h"



 /*
 ready_to_read

 Waits until a socket is ready to read
 */
 int ready_to_read(int s)
 {
 		fd_set readfds;
 		struct timeval tv;
 		
 		FD_ZERO(&readfds);
 		FD_SET(s, &readfds);
 		
 		tv.tv_sec = R_GIVEUP_SEC;
 		tv.tv_usec = R_GIVEUP_USEC;
 		
 		select(s+1, &readfds, NULL, NULL, &tv);
 		
 		if (FD_ISSET(s, &readfds))
 			return 1;
 			
 		return 0;
 }


 /*
 receive_message

 This function receives a message and puts it in a buffer.
 The function allocates the buffer itself so when we're done
 with it, we need to call a cleanup function.
 */
 char *receive_message(int s)
 {
 		unsigned long length, *pL;
 		char *buf, *pC=(char *) &length;
 		int size = 4, rec_size;
 		
 		while (size>0) {
 			if (ready_to_read(s))
 				rec_size = recv(s, pC, size, 0);
 			else
 				return NULL;
 			if (rec_size == 0)
 				return NULL;
 			if (rec_size == -1)
 				return NULL;
 			pC += rec_size;
 			size -= rec_size;
 		}

#ifdef DEBUG
			fprintf(stderr, "Going to allocate %lu bytes\n", ntohl(length));
#endif
 		
 		buf = malloc(ntohl(length));
 		pL = (unsigned long *) buf;
 		
 		if (!buf) return NULL;
 		
 		*pL = length;
 		pC = buf+4;
 		length = ntohl(length);
 		length -= 4;
 		
 		while (length>0) {
 			if (ready_to_read(s))
 				rec_size = recv(s, pC, length, 0);
 			else
 				return NULL;
 			if (rec_size == 0)
 				return NULL;
 			if (rec_size == -1)
 				return NULL;
 			pC += rec_size;
 			length -= rec_size;
 		}
 		
 		return buf;
 }

 /*
 clean_buf

 This function frees buffers allocated by the receive_message function.
 */
 void clean_buf(char *buf)
 {
 		free(buf);
 		return;
 }

 /*
 message_type

 This function returns the mesage type of the message stored in the buffer
 pointed to by the given pointer.
 */
 unsigned long message_type(char *msg)
 {
 		unsigned long *pUL=(unsigned long *) (msg+4);
 		return ntohl(*pUL);
 }


 /*
 ready_to_write

 Waits until a socket is ready to write
 */
 int ready_to_write(int s)
 {
 		fd_set writefds;
 		struct timeval tv;
 		
 		FD_ZERO(&writefds);
 		FD_SET(s, &writefds);
 		
 		tv.tv_sec = W_GIVEUP_SEC;
 		tv.tv_usec = W_GIVEUP_USEC;
 		
 		select(s+1, NULL, &writefds, NULL, &tv);
 		
 		if (FD_ISSET(s, &writefds))
 			return 1;
 			
 		return 0;
 }



 /*
 send_message

 This function sends a message over a socket.
 */
 int send_message(int s, char *msg, unsigned long len)
 {
 		int send_size;
 		
 		while (len>0)
 		{
 			if (ready_to_write(s))
 				send_size = send(s, msg, len, 0);
 			else
 				return -1;
 			if (send_size == -1)
 				return -1;
 			if (send_size == 0)
 				return -1;
 			msg += send_size;
 			len -= send_size;
 		}
 		
 		return 0;
 }

 /*
 announce_connection

 Informs of an incoming connection
 */
 int announce_connection(struct sockaddr_in *remote_addr)
 {
#ifdef DO_LOG
		struct in_addr connected_addr;
		connected_addr.s_addr = remote_addr->sin_addr.s_addr;
 		fprintf(LOG_FILE, ">>New connection from %s:%d\n", inet_ntoa(connected_addr), ntohs(remote_addr->sin_port));
#endif

 		return 0;
 }
